==> tmdy_dll_0.0.0-031117.zip: TiMidity++-2.12.0-pre1b-031117-0200/libarc/url_inflate.c <==
/*
    TiMidity++ -- MIDI to WAVE converter and player
    Copyright (C) 1999-2002 Masanao Izumo <mo@goice.co.jp>
    Copyright (C) 1995 Tuukka Toivonen <tt@cgs.fi>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */
#include <stdio.h>
#include <stdlib.h>
#include "timidity.h"
#include "url.h"
#include "url_prv.h"

#include "mblock.h"
#include "zip.h"

typedef struct _URL_inflate
{
    char common[sizeof(struct _URL)];
    InflateHandler decoder;
    URL instream;
    long compsize;
    long pos;
    int autoclose;
} URL_inflate;

static long url_inflate_read_func(tmdy_struct_ex_t *tmdy_struct, char *buf, long size, void *v);
static long url_inflate_read(tmdy_struct_ex_t *tmdy_struct, URL url, void *buff, long n);
static long url_inflate_tell(tmdy_struct_ex_t *tmdy_struct, URL url);
static void url_inflate_close(tmdy_struct_ex_t *tmdy_struct, URL url);

URL url_inflate_open(tmdy_struct_ex_t *tmdy_struct, URL instream, long compsize, int autoclose)
{
    URL_inflate *url;

    url = (URL_inflate *)alloc_url(tmdy_struct, sizeof(URL_inflate));
    if(url == NULL)
    {
	if(autoclose)
	    url_close(tmdy_struct, instream);
	url_errno = errno;
	return NULL;
    }

    /* common members */
    URLm(url, type)      = URL_inflate_t;
    URLm(url, url_read)  = url_inflate_read;
    URLm(url, url_gets)  = NULL;
    URLm(url, url_fgetc) = NULL;
    URLm(url, url_seek)  = NULL;
    URLm(url, url_tell)  = url_inflate_tell;
    URLm(url, url_close) = url_inflate_close;

    /* private members */
    url->decoder = NULL;
    url->instream = instream;
    url->pos = 0;
    url->compsize = compsize;
    url->autoclose = autoclose;

    errno = 0;
    url->decoder = TMDY_ARC->zip->open_inflate_handler(tmdy_struct, url_inflate_read_func, url);
    if(url->decoder == NULL)
    {
	if(autoclose)
	    url_close(tmdy_struct, instream);
	url_inflate_close(tmdy_struct, (URL)url);
	url_errno = errno;
	return NULL;
    }

    return (URL)url;
}

static long url_inflate_read_func(tmdy_struct_ex_t *tmdy_struct, char *buf, long size, void *v)
{
    URL_inflate *urlp = (URL_inflate *)v;
    long n;

    if(urlp->compsize == -1) /* size if unknown */
	return url_read(tmdy_struct, urlp->instream, buf, size);

    if(urlp->compsize == 0)
	return 0;
    n = size;
    if(n > urlp->compsize)
	n = urlp->compsize;
    n = url_read(tmdy_struct, urlp->instream, buf, n);
    if(n == -1)
	return -1;
    urlp->compsize -= n;
    return n;
}

static long url_inflate_read(tmdy_struct_ex_t *tmdy_struct, URL url, void *buff, long n)
{
    URL_inflate *urlp = (URL_inflate *)url;

    n = TMDY_ARC->zip->zip_inflate(tmdy_struct, urlp->decoder, (char *)buff, n);
    if(n <= 0)
	return n;
    urlp->pos += n;
    return n;
}

static long url_inflate_tell(tmdy_struct_ex_t *tmdy_struct, URL url)
{
    return ((URL_inflate *)url)->pos;
}

static void url_inflate_close(tmdy_struct_ex_t *tmdy_struct, URL url)
{
    int save_errno = errno;
    URL_inflate *urlp = (URL_inflate *)url;
    if(urlp->decoder)
	TMDY_ARC->zip->close_inflate_handler(tmdy_struct, urlp->decoder);
    if(urlp->autoclose)
	url_close(tmdy_struct, urlp->instream);
    free(url);
    errno = save_errno;
}

==> tmdy_dll_0.0.0-031125.zip: TiMidity++-2.12.0-pre1b-031117-0200/libarc/url_inflate.c <==
/*
    TiMidity++ -- MIDI to WAVE converter and player
    Copyright (C) 1999-2002 Masanao Izumo <mo@goice.co.jp>
    Copyright (C) 1995 Tuukka Toivonen <tt@cgs.fi>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */
#include <stdio.h>
#include <stdlib.h>
#include "timidity.h"
#include "url.h"
#include "url_prv.h"

#include "mblock.h"
#include "zip.h"

typedef struct s_URL_inflate
{
    char common[sizeof(struct s_URL)];
    InflateHandler decoder;
    URL instream;
    long compsize;
    long pos;
    int autoclose;
} URL_inflate;

static long url_inflate_read_func(tmdy_struct_ex_t *tmdy_struct, char *buf, long size, void *v);
static long url_inflate_read(tmdy_struct_ex_t *tmdy_struct, URL url, void *buff, long n);
static long url_inflate_tell(tmdy_struct_ex_t *tmdy_struct, URL url);
static void url_inflate_close(tmdy_struct_ex_t *tmdy_struct, URL url);

URL url_inflate_open(tmdy_struct_ex_t *tmdy_struct, URL instream, long compsize, int autoclose)
{
    URL_inflate *url;

    url = (URL_inflate *)alloc_url(tmdy_struct, sizeof(URL_inflate));
    if(url == NULL)
    {
	if(autoclose)
	    url_close(tmdy_struct, instream);
	url_errno = errno;
	return NULL;
    }

    /* common members */
    URLm(url, type)      = URL_inflate_t;
    URLm(url, url_read)  = url_inflate_read;
    URLm(url, url_gets)  = NULL;
    URLm(url, url_fgetc) = NULL;
    URLm(url, url_seek)  = NULL;
    URLm(url, url_tell)  = url_inflate_tell;
    URLm(url, url_close) = url_inflate_close;

    /* private members */
    url->decoder = NULL;
    url->instream = instream;
    url->pos = 0;
    url->compsize = compsize;
    url->autoclose = autoclose;

    errno = 0;
    url->decoder = TMDY_ARC->zip->open_inflate_handler(tmdy_struct, url_inflate_read_func, url);
    if(url->decoder == NULL)
    {
	if(autoclose)
	    url_close(tmdy_struct, instream);
	url_inflate_close(tmdy_struct, (URL)url);
	url_errno = errno;
	return NULL;
    }

    return (URL)url;
}

static long url_inflate_read_func(tmdy_struct_ex_t *tmdy_struct, char *buf, long size, void *v)
{
    URL_inflate *urlp = (URL_inflate *)v;
    long n;

    if(urlp->compsize == -1) /* size if unknown */
	return url_read(tmdy_struct, urlp->instream, buf, size);

    if(urlp->compsize == 0)
	return 0;
    n = size;
    if(n > urlp->compsize)
	n = urlp->compsize;
    n = url_read(tmdy_struct, urlp->instream, buf, n);
    if(n == -1)
	return -1;
    urlp->compsize -= n;
    return n;
}

static long url_inflate_read(tmdy_struct_ex_t *tmdy_struct, URL url, void *buff, long n)
{
    URL_inflate *urlp = (URL_inflate *)url;

    n = TMDY_ARC->zip->zip_inflate(tmdy_struct, urlp->decoder, (char *)buff, n);
    if(n <= 0)
	return n;
    urlp->pos += n;
    return n;
}

static long url_inflate_tell(tmdy_struct_ex_t *tmdy_struct, URL url)
{
    return ((URL_inflate *)url)->pos;
}

static void url_inflate_close(tmdy_struct_ex_t *tmdy_struct, URL url)
{
    int save_errno = errno;
    URL_inflate *urlp = (URL_inflate *)url;
    if(urlp->decoder)
	TMDY_ARC->zip->close_inflate_handler(tmdy_struct, urlp->decoder);
    if(urlp->autoclose)
	url_close(tmdy_struct, urlp->instream);
    free(url);
    errno = save_errno;
}
